# Copyright 2014 - Rackspace
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import unittest

from solumdashboard.common import workflow_parsers


MISTRAL_DSL = {
    "Namespaces": {
        "Nova": {
            "actions": "random data"
        }
    },
    "Workflow": {
        "tasks": {
            "createVM": {
                "action": "Nova.createVM",
                "parameters": {
                    "nova_url": "$.nova_url",
                    "auth_token": "$.auth_token"
                },
                "publish": {
                    "vm_id": "vm_id"
                },
                "on-success": "waitForIP"
            },
            "waitForIP": {
                "action": "Nova.getIP",
                "publish": {
                    "vm_ip": "vm_ip"
                },
                "parameters": {
                    "nova_url": "$.nova_url",
                    "project_id": "$.project_id",
                    "auth_token": "$.auth_token"
                },
                "on-error": "errorTask"
            }
        }
    }
}

HEAT_TEMPLATE = {
    "heat_template_version": "2013-05-23",
    "description": "Basic app deploy.",
    "parameters": {
        "app_name": {
            "type": "string",
            "default": "solum-app",
            "description": "app name desc"
        },
        "key_name": {
            "type": "string",
            "default": "mykey"
        }
    },
    "randomstuff": "blah blah blah"
}


class TestWorkflowParsers(unittest.TestCase):
    """Test routines for workflow parsing code."""
    def test_get_mistral_tasks(self):
        """Test get Mistral Tasks functionality."""
        # Correct operation test case
        task_list = workflow_parsers.get_mistral_tasks(MISTRAL_DSL)
        self.assertEqual(task_list,
                         [["createVM", "waitForIP", None],
                          ["waitForIP", None, "errorTask"]])
        # Missing "tasks" section from Workflow
        dsl_copy = MISTRAL_DSL.copy()
        dsl_copy["Workflow"] = {"garbage": "here", "nothing": "useful"}
        self.assertRaises(KeyError, workflow_parsers.get_mistral_tasks,
                          dsl_copy)
        # All tasks have an on-success condition (invalid DSL)
        dsl_copy = MISTRAL_DSL.copy()
        dsl_copy["Workflow"]["tasks"]["waitForIP"]["on-success"] = "noTask"
        self.assertRaises(IndexError, workflow_parsers.get_mistral_tasks,
                          dsl_copy)
        # None data test case
        self.assertRaises(TypeError, workflow_parsers.get_mistral_tasks, None)

    def test_get_mistral_required_input(self):
        """Test get Mistral required user input functionality."""
        # Handle None data
        self.assertRaises(TypeError, workflow_parsers.get_mistral_tasks, None)
        # Correct operation test case
        input_dict = workflow_parsers.get_mistral_required_input(MISTRAL_DSL)
        self.assertEqual(input_dict,
                         {"nova_url": ["createVM", "waitForIP"],
                          "auth_token": ["createVM", "waitForIP"],
                          "project_id": ["waitForIP"]})
        # Remove the "tasks" section from Workflow
        dsl_copy = MISTRAL_DSL.copy()
        dsl_copy["Workflow"] = {"garbage": "here", "nothing": "useful"}
        self.assertRaises(KeyError,
                          workflow_parsers.get_mistral_required_input,
                          dsl_copy)
        # Verify that a publish removes the input from the next task
        dsl_copy = MISTRAL_DSL.copy()
        dsl_copy["Workflow"]["tasks"]["createVM"]["publish"] = (
            {"project_id": "test project"})
        input_dict = workflow_parsers.get_mistral_required_input(dsl_copy)
        self.assertEqual(input_dict,
                         {"nova_url": ["createVM", "waitForIP"],
                          "auth_token": ["createVM", "waitForIP"]})
        # Test ignoring of odd tags like "arguments"
        dsl_copy["Workflow"]["tasks"]["createVM"]["parameters"] = (
            {"key": "value", "arguments": {"random arguments"}})
        input_dict = workflow_parsers.get_mistral_required_input(dsl_copy)
        self.assertEqual(input_dict,
                         {"key": ["createVM"], "nova_url": ["waitForIP"],
                          "auth_token": ["waitForIP"]})

    def test_get_heat_required_input(self):
        """Test get Heat required user input functionality."""
        # Handle None data
        self.assertRaises(TypeError, workflow_parsers.get_heat_required_input,
                          None)
        # Remove the "parameters" section from the HoT
        hot_copy = {"nothing_here": "error time"}
        self.assertRaises(KeyError,
                          workflow_parsers.get_heat_required_input,
                          hot_copy)
        # Correct operation test case
        heat_list = workflow_parsers.get_heat_required_input(HEAT_TEMPLATE)
        self.assertEqual(heat_list,
                         [["app_name", "string", "solum-app", "app name desc"],
                          ["key_name", "string", "mykey", None]])


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2014 Rackspace Hosting.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.urls import reverse
from django.urls import reverse_lazy
from django.utils.translation import ugettext_lazy as _

import json

from horizon import exceptions
from horizon import forms
from horizon import tables
from horizon import views

from solumclient.v1 import languagepack as cli_lp

from solumdashboard.api.client import client as solumclient
from solumdashboard.languagepacks import forms as lp_forms
from solumdashboard.languagepacks import tables as lp_tables


class IndexView(tables.DataTableView):
    table_class = lp_tables.LanguagepacksTable
    template_name = 'languagepacks/index.html'
    page_title = _("Languagepacks")

    def get_data(self):
        try:
            solum = solumclient(self.request)
            languagepacks = solum.languagepacks.list()
        except Exception as e:
            languagepacks = []
            exceptions.handle(
                self.request,
                _('Unable to retrieve languagepacks: %s') % e)
        return languagepacks


class DetailView(views.HorizonTemplateView):
    template_name = 'languagepacks/detail.html'
    page_title = "{{ languagepack.name }}"

    def get_context_data(self, **kwargs):
        context = super(DetailView, self).get_context_data(**kwargs)

        languagepack, loglist = self.get_data()
        context["languagepack"] = languagepack
        context["loglist"] = loglist
        table = lp_tables.LanguagepacksTable(self.request)
        context["actions"] = table.render_row_actions(languagepack)
        return context

    def get_data(self):
        lp_id = self.kwargs['languagepack_id']
        solum = solumclient(self.request)
        languagepack = None
        loglist = []
        try:
            languagepack = solum.languagepacks.find(name_or_id=lp_id)
            loglist = cli_lp.LanguagePackManager(solum).logs(
                lp_id=lp_id)
        except Exception as e:
            INDEX_URL = 'horizon:solum:languagepacks:index'
            exceptions.handle(
                self.request,
                _('Unable to retrieve languagepack details: %s') % e,
                redirect=reverse(INDEX_URL))

        for log in loglist:
            strategy_info = json.loads(log.strategy_info)
            if log.strategy == 'local':
                log.local_storage = log.location
            elif log.strategy == 'swift':
                log.swift_container = strategy_info['container']
                log.swift_path = log.location

        return languagepack, loglist


class CreateView(forms.ModalFormView):
    form_class = lp_forms.CreateForm
    template_name = 'languagepacks/create.html'
    modal_header = _("Create Languagepack")
    page_title = _("Create Languagepack")
    submit_url = reverse_lazy("horizon:solum:languagepacks:create")
    success_url = reverse_lazy("horizon:solum:languagepacks:index")



# Copyright (c) 2014 Rackspace Hosting.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ungettext_lazy

from horizon import tables
from horizon.utils import filters

from solumclient.v1 import app as cli_app

from solumdashboard.api.client import client as solumclient
from solumdashboard.applications import tabs


class CreateApplication(tables.LinkAction):
    name = "create"
    verbose_name = _("New Application")
    url = "horizon:solum:applications:create"
    classes = ("btn-launch", "ajax-modal")


class ScaleApplication(tables.LinkAction):
    name = "scale"
    verbose_name = _("Scale Application")
    url = "horizon:solum:applications:scale"
    classes = ("btn-launch", "ajax-modal")


class UpdateApplication(tables.LinkAction):
    name = "update"
    verbose_name = _("Update Application")
    url = "horizon:solum:applications:update"
    classes = ("btn-launch", "ajax-modal")


class LaunchApplication(tables.LinkAction):
    name = "build-and-deploy"
    verbose_name = _("Build and Deploy Application")
    url = "horizon:solum:applications:launch"
    classes = ("btn-launch", "ajax-modal")

    def allowed(self, request, package):
        return True


class DeleteApplication(tables.DeleteAction):
    name = "delete"

    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Delete Application",
            u"Delete Applications",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Deleted Application",
            u"Deleted Applications",
            count
        )

    def allowed(self, request, template):
        return True

    def delete(self, request, application_id):
        solum = solumclient(request)
        cli_app.AppManager(solum).delete(app_id=application_id)


class ViewApplicationLogs(tables.LinkAction):
    name = "view"
    verbose_name = _("View Application Logs")
    url = "horizon:solum:applications:detail"
    classes = ("btn-edit",)

    def get_link_url(self, datum):
        base_url = super(ViewApplicationLogs, self).get_link_url(datum)
        tab_query_string = tabs.LogTab(
            tabs.AppDetailsTabs).get_query_string()
        return "?".join([base_url, tab_query_string])


class ApplicationsTable(tables.DataTable):
    name = tables.Column('name', verbose_name=_('Name'))
    id = tables.Column('id', verbose_name=_('ID'),
                       link=("horizon:solum:applications:detail"))
    created_at = tables.Column('created_at', verbose_name=_('Created at'),
                               filters=(filters.parse_isotime,))
    description = tables.Column('description', verbose_name=_('Description'))
    languagepack = tables.Column('languagepack',
                                 verbose_name=_('Languagepack'))

    def get_object_id(self, app):
        return app.id

    class Meta(object):
        name = "applications"
        verbose_name = _("Applications")
        table_actions = (CreateApplication, DeleteApplication)
        row_actions = (LaunchApplication, ViewApplicationLogs,
                       DeleteApplication, ScaleApplication, UpdateApplication)



# Copyright (c) 2014 Rackspace Hosting.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import tables
from horizon import tabs

from solumdashboard.api.client import client as solumclient
from solumdashboard.assemblies import tables as assem_tables
import solumdashboard.assemblies.tabs as _tabs


class IndexView(tables.DataTableView):
    table_class = assem_tables.AssembliesTable
    template_name = 'assemblies/index.html'
    page_title = _("Assemblies")

    def get_data(self):
        try:
            solum = solumclient(self.request)
            assemblies = solum.assemblies.list()
        except Exception as e:
            assemblies = []
            exceptions.handle(
                self.request,
                _('Unable to retrieve assemblies: %s') % e)
        return assemblies


class DetailView(tabs.TabView):
    template_name = 'assemblies/detail.html'
    tab_group_class = _tabs.AssemDetailsTabs
    page_title = "{{ assem.name|default:assem.uuid }}"

    def get_context_data(self, **kwargs):
        context = super(DetailView, self).get_context_data(**kwargs)
        assem_id = kwargs['assembly_id']
        solum = solumclient(self.request)
        assem = solum.assemblies.get(assembly_id=assem_id)
        context["assem"] = assem
        return context

    def get_data(self):
        pass



# Copyright (c) 2014 Rackspace Hosting.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.urls import reverse
from django.urls import reverse_lazy
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import tables
from horizon import tabs
from horizon import workflows

from solumdashboard.api.client import client as solumclient
from solumdashboard.applications import forms as app_forms
from solumdashboard.applications import tables as app_tables
import solumdashboard.applications.tabs as _tabs
from solumdashboard.applications import workflows as update_flow


class IndexView(tables.DataTableView):
    table_class = app_tables.ApplicationsTable
    template_name = 'applications/index.html'
    page_title = _("Applications")

    def get_data(self):
        try:
            solum = solumclient(self.request)
            apps = solum.apps.list()
        except Exception as e:
            apps = []
            exceptions.handle(
                self.request,
                _('Unable to retrieve apps: %s') % e)
        return apps


class CreateView(forms.ModalFormView):
    form_class = app_forms.CreateForm
    template_name = 'applications/create.html'
    modal_header = _("Create Application")
    page_title = _("Create Application")
    submit_url = reverse_lazy('horizon:solum:applications:create')
    success_url = reverse_lazy("horizon:solum:applications:index")


class ScaleView(forms.ModalFormView):
    form_class = app_forms.ScaleForm
    template_name = "applications/scale.html"
    modal_header = _("Scale Application")
    page_title = _("Scale Application")
    submit_url = reverse_lazy('horizon:solum:applications:scale')
    success_url = reverse_lazy("horizon:solum:applications:index")
    failure_url = reverse_lazy("horizon:solum:applications:index")

    def get_context_data(self, **kwargs):
        context = super(ScaleView, self).get_context_data(**kwargs)
        context["application_id"] = self.kwargs["application_id"]
        return context

    def get_initial(self):
        application_id = self.kwargs['application_id']
        return {'application_id': application_id}


class UpdateView(workflows.WorkflowView):
    workflow_class = update_flow.UpdateApplicationClass
    success_url = "horizon:solum:applications:index"
    classes = ("ajax-modal")

    def get_context_data(self, **kwargs):
        context = super(UpdateView, self).get_context_data(**kwargs)
        context["application_id"] = self.kwargs["application_id"]
        return context

    def _get_object(self, *args, **kwargs):
        application_id = self.kwargs['application_id']
        solum = solumclient(self.request)
        app = solum.apps.find(name_or_id=application_id)
        return app

    def get_initial(self):
        app = self._get_object()
        return {'application_id': app.id}


class DetailView(tabs.TabView):
    template_name = 'applications/detail.html'
    tab_group_class = _tabs.AppDetailsTabs
    page_title = "{{ app.name|default:app.id }}"

    def get_context_data(self, **kwargs):
        context = super(DetailView, self).get_context_data(**kwargs)
        application_id = self.kwargs['application_id']
        app = None
        try:
            solum = solumclient(self.request)
            app = solum.apps.find(name_or_id=application_id)
        except Exception as e:
            INDEX_URL = 'horizon:solum:applications:index'
            exceptions.handle(
                self.request,
                _('Unable to retrieve application details: %s') % str(e),
                redirect=reverse(INDEX_URL))
        context["app"] = app
        table = app_tables.ApplicationsTable(self.request)
        context["actions"] = table.render_row_actions(app)
        return context


class LaunchView(forms.ModalFormView):
    form_class = app_forms.LaunchForm
    template_name = "applications/launch.html"
    modal_header = _("Launch Application")
    page_title = _("Launch Application")
    success_url = reverse_lazy("horizon:solum:applications:index")
    failure_url = reverse_lazy("horizon:solum:applications:index")

    def get_context_data(self, **kwargs):
        context = super(LaunchView, self).get_context_data(**kwargs)
        context["application_id"] = self.kwargs["application_id"]
        return context

    def get_initial(self):
        application_id = self.kwargs['application_id']
        return {'application_id': application_id}



# Copyright (c) 2014 Rackspace Hosting.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.utils.translation import ugettext_lazy as _

import json

from horizon import tabs

from solumclient.v1 import workflow as cli_wf

from solumdashboard.api.client import client as solumclient


class GeneralTab(tabs.Tab):
    name = _("General Info")
    slug = "application_details_tab"
    template_name = ("applications/_detail.html")

    def get_context_data(self, request):
        app_id = self.tab_group.kwargs['application_id']
        solum = solumclient(request)
        app = solum.apps.find(name_or_id=app_id)

        app.trigger = app.trigger_actions
        app.workflow = app.workflow_config
        if app.scale_config:
            app.target_instances = app.scale_config[app.name].get('target', '')

        workflowman = cli_wf.WorkflowManager(solum, app_id=app_id)
        workflows = workflowman.list()

        return {"application": app, "workflows": workflows}


class LogTab(tabs.Tab):
    name = _("Logs")
    slug = "application_logs_tab"
    template_name = ("applications/_log.html")

    def get_context_data(self, request):
        app_id = self.tab_group.kwargs['application_id']
        solum = solumclient(request)
        app = solum.apps.find(name_or_id=app_id)

        workflowman = cli_wf.WorkflowManager(solum, app_id=app_id)
        workflows = workflowman.list()

        logs_list = []

        for workflow in workflows:
            revision = workflow.wf_id
            loglist = workflowman.logs(revision_or_id=revision)
            for log in loglist:
                logs_list.append(log)
                strategy_info = json.loads(log.strategy_info)
                if log.strategy == 'local':
                    log.local_storage = log.location
                elif log.strategy == 'swift':
                    log.swift_container = strategy_info['container']
                    log.swift_path = log.location

        return {"logs": logs_list, "application": app}


class AppDetailsTabs(tabs.TabGroup):
    slug = "application_details"
    tabs = (GeneralTab, LogTab)
    sticky = True



# Copyright (c) 2014 Rackspace Hosting.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ungettext_lazy

from horizon import tables

from solumdashboard.api.client import client as solumclient


class DeleteLanguagepack(tables.DeleteAction):
    name = "delete"

    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Delete Languagepack",
            u"Delete Languagepacks",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Deleted Languagepack",
            u"Deleted Languagepacks",
            count
        )

    def allowed(self, request, template):
        return True

    def delete(self, request, languagepack_id):
        solum = solumclient(request)
        solum.languagepacks.delete(lp_id=languagepack_id)


class UpdateRow(tables.Row):
    ajax = True

    def get_data(self, request, languagepack_id):
        solum = solumclient(request)
        return solum.languagepacks.find(name_or_id=languagepack_id)


class CreateLanguagepack(tables.LinkAction):
    name = "create"
    verbose_name = _("New Languagepack")
    url = "horizon:solum:languagepacks:create"
    classes = ("btn-launch", "ajax-modal")


class LanguagepacksTable(tables.DataTable):
    STATUS_CHOICES = (
        ("QUEUED", None),
        ("BUILDING", None),
        ("ERROR", False),
        ("READY", True),
    )
    uuid = tables.Column("uuid", verbose_name=_("UUID"),
                         link=("horizon:solum:languagepacks:detail"))
    name = tables.Column("name", verbose_name=_("Name"))
    description = tables.Column("description", verbose_name=_("Description"))
    status = tables.Column("status", verbose_name=_("Status"),
                           status=True,
                           status_choices=STATUS_CHOICES)
    source_uri = tables.Column("source_uri", verbose_name=_("Source Uri"))

    def get_object_id(self, lp):
        return lp.uuid

    class Meta(object):
        name = "languagepacks"
        verbose_name = _("Languagepacks")
        row_class = UpdateRow
        status_columns = ["status"]
        table_actions = (CreateLanguagepack, DeleteLanguagepack)
        row_actions = (DeleteLanguagepack,)
